# -*- coding: utf-8 -*-
from odoo import models, fields, api, _
from odoo.exceptions import ValidationError
from odoo.http import request


class EventType(models.Model):
    _inherit = "event.type"

    def _default_question_ids(self):
        # ask for company's name by default
        return super()._default_question_ids() + [
            (0, 0, {'title': _('Company'), 'question_type': 'company_name', 'is_mandatory_answer': True}),
        ]

    question_ids = fields.One2many(default=lambda s: s._default_question_ids())


# Extension for events to add customizable part for registration page (banner, roadshow info, ...)
class Event(models.Model):
    _inherit = "event.event"

    # Reintroduce useful field for sharing mail templates across multiple events.
    # Dropped in 11.0, cfr task #32484 or commit b1642f87101a794adc6fd3a3bad0a283e3260b69
    reply_to = fields.Char(
        'Contact Email', readonly=False,
        help="The email address of the organizer is likely to be put here, with the effect to be in the 'Reply-To' of the mails sent automatically at event or registrations confirmation. You can also put the email address of your mail gateway if you use one.")

    custom_banner_url = fields.Char("Banner URL") # Default for roadshow is defined in view
    custom_roadshow_column_content = fields.Html(
        "Roadshow column content",
        compute="_compute_custom_roadshow_column_content", # TODO computation should be replaced by default t-field content in the view
        inverse="_set_custom_roadshow_column_content",
        store=True)

    is_roadshow = fields.Boolean("Is Roadshow", compute="_compute_is_roadshow")

    @api.depends("event_type_id")
    def _compute_custom_roadshow_column_content(self):
        for event in self:
            content = event.custom_roadshow_column_content
            if event.is_roadshow:
                if not content:
                    template_name = "openerp_website.default_custom_roadshow_column_content"
                    template = self.env.ref(template_name, raise_if_not_found=False)
                    content = self.env['ir.qweb']._render(template_name, raise_if_not_found=False) if template else False
            elif content:
                content = False
            event.custom_roadshow_column_content = content

    def _set_custom_roadshow_column_content(self):
        pass

    def _compute_is_roadshow(self):
        roadshow_event_type = self.env.ref("openerp_website.roadshow_event_type", raise_if_not_found=False)
        for event in self:
            event.is_roadshow = (roadshow_event_type and event.event_type_id.id == roadshow_event_type.id)

    @api.constrains('name')
    def _contrains_name(self):
        # The event name cannot contain a return line because the name is used as both the subject and attachment name.
        # This would render the email headers invalid and cause the email to crash. If there are newline characters ('\n')
        # in the subject, the Python standard library function check len(splitlines()) == 1 that will always return True
        # if the \n is the last char, causing the email to be sent without warning. However, most email servers will reject it.
        for rec in self:
            if '\n' in rec.name:
                raise ValidationError('The name of the event cannot contain return line. [%s]' % rec)

# Temporary extension for Odoo Experience 2016, can be safely dropped later
class event_registration(models.Model):
    _inherit = "event.registration"

    invoice_ids = fields.Many2many(
        'account.move', 'Invoices', related='sale_order_id.invoice_ids', copy=False)
    invoice_count = fields.Integer('Invoices #', related='sale_order_id.invoice_count')

    @api.model
    def _default_country(self):
        country = self.env['res.country']
        if request and hasattr(request, 'website'):
            # automatically set default country for frontend requests
            country_code = request.geoip.country_code
            country = country_code and request.env['res.country'].search([('code', '=', country_code)]) or country
        return country

    @api.constrains('partner_id')
    def _onchange_partner(self):
        if self.partner_id:
            partner_is_public = any([u._is_public() for u in self.partner_id.with_context(active_test=False).user_ids])
            if not partner_is_public:
                self.country_id = self.partner_id.country_id

    country_id = fields.Many2one('res.country', string='Country', default=_default_country)

    def action_view_invoice(self):
        return self.sale_order_line_id.order_id.action_view_invoice()

    def _get_lead_contact_values(self):
        """ overide method from event_crm to add country on the lead value """
        contact_vals = super()._get_lead_contact_values()
        country_id = False
        if contact_vals.get('partner_id'):
            country_id = self.env['res.partner'].browse(contact_vals['partner_id']).country_id.id
        if not country_id:
            country_id = self._find_first_notnull('country_id')

        if country_id:
            contact_vals['country_id'] = country_id
        return contact_vals


class event_track(models.Model):
    _inherit = "event.track"

    lang_id = fields.Many2one('res.lang', 'Language', default=lambda self: self.env['res.lang']._lang_get(self.env.user.lang))
